﻿param($rootPath, $toolsPath, $package, $project)

$scLabel = "WebCompiler"

# When this package is installed we need to add a property
# to the current project, WebCompilerTargets, which points to the
# .targets file in the packages folder

function RemoveExistingWebCompilerPropertyGroups($projectRootElement){
    # if there are any PropertyGroups with a label of "WebCompiler" they will be removed here
    $pgsToRemove = @()
    foreach($pg in $projectRootElement.PropertyGroups){
        if($pg.Label -and [string]::Compare($scLabel,$pg.Label,$true) -eq 0) {
            # remove this property group
            $pgsToRemove += $pg
        }
    }

    foreach($pg in $pgsToRemove){
        $pg.Parent.RemoveChild($pg)
    }
}

# TODO: Revisit this later, it was causing some exceptions
function CheckoutProjFileIfUnderScc(){
    # http://daltskin.blogspot.com/2012/05/nuget-powershell-and-tfs.html
    $sourceControl = Get-Interface $project.DTE.SourceControl ([EnvDTE80.SourceControl2])
    if($sourceControl.IsItemUnderSCC($project.FullName) -and $sourceControl.IsItemCheckedOut($project.FullName)){
        $sourceControl.CheckOutItem($project.FullName)
    }
}

function EnsureProjectFileIsWriteable(){
    $projItem = Get-ChildItem $project.FullName
    if($projItem.IsReadOnly) {
        "The project file is read-only. Please checkout the project file and re-install this package" | Write-Host -ForegroundColor Red
        throw;
    }
}

function ComputeRelativePathToTargetsFile(){
    param($startPath,$targetPath)

    # we need to compute the relative path
    $startLocation = Get-Location

    Set-Location $startPath.Directory | Out-Null
    $relativePath = Resolve-Path -Relative $targetPath.FullName

    # reset the location
    Set-Location $startLocation | Out-Null

    return $relativePath
}

function GetSolutionDirFromProj{
    param($msbuildProject)

    if(!$msbuildProject){
        throw "msbuildProject is null"
    }

    $result = $null
    $solutionElement = $null
    foreach($pg in $msbuildProject.PropertyGroups){
        foreach($prop in $pg.Properties){
            if([string]::Compare("SolutionDir",$prop.Name,$true) -eq 0){
                $solutionElement = $prop
                break
            }
        }
    }

    if($solutionElement){
        $result = $solutionElement.Value
    }

    return $result
}

function AddImportElementIfNotExists(){
    param($projectRootElement)

    $foundImport = $false
    $importsToRemove = @()
    foreach($import in $projectRootElement.Imports){
        $importStr = $import.Project
        if(!$importStr){
            $importStr = ""
        }

        if([string]::Compare('$(WebCompilerTargets)',$importStr.Trim(),$true) -eq 0){
            if(!$foundImport){
               # if it doesn't have a label then add one
                if([string]::IsNullOrWhiteSpace($import.Label)){
                    $import.Label = $scLabel
                }
                $import.Condition="Exists('`$(WebCompilerTargets)')"

                $foundImport = $true
            }
            else{
                # if we already found an import, this must be a duplicate remove it
                $importsToRemove+=$import
            }
        }
    }

    foreach($import in $importsToRemove){
        # $projectRootElement.Imports.Remove($import)
        # you have to use Microsoft.Build.Evaluation.ProjectCollection to remove, so disabling should be good enough
        $import.Condition='false'
    }

    if(!$foundImport){
        # the import is not in the project, add it
        # <Import Project="$(WebCompilerTargets)" Condition="Exists('$(WebCompilerTargets)')" Label="WebCompiler" />
        $importToAdd = $projectRootElement.AddImport('$(WebCompilerTargets)');
        $importToAdd.Condition = "Exists('`$(WebCompilerTargets)')"
        $importToAdd.Label = $scLabel
    }
}



#########################
# Start of script here
#########################

$projFile = $project.FullName

# Make sure that the project file exists
if(!(Test-Path $projFile)){
    throw ("Project file not found at [{0}]" -f $projFile)
}

# use MSBuild to load the project and add the property

# This is what we want to add to the project
#  <PropertyGroup Label="WebCompiler">
#      <WebCompiler_EnableImportFromNuGet Condition=" '$(SC_EnableImportFromNuGet)'=='' ">true</WebCompiler_EnableImportFromNuGet>
#      <WebCompiler_NuGetImportPath Condition=" '$(WebCompiler_NuGetImportPath)'=='' ">$([System.IO.Path]::GetFullPath( $(Filepath) ))</WebCompiler_NuGetImportPath>
#      <WebCompilerTargets Condition=" '$(WebCompiler_EnableImportFromNuGet)'=='true' and Exists('$(WebCompiler_NuGetImportPath)') ">$(WebCompiler_NuGetImportPath)</WebCompilerTargets>
#  </PropertyGroup>


# EnsureProjectFileIsWriteable
# Before modifying the project save everything so that nothing is lost
$DTE.ExecuteCommand("File.SaveAll")
CheckoutProjFileIfUnderScc
EnsureProjectFileIsWriteable

# Update the Project file to import the .targets file

$projectMSBuild = [Microsoft.Build.Construction.ProjectRootElement]::Open($projFile)

RemoveExistingWebCompilerPropertyGroups -projectRootElement $projectMSBuild
$propertyGroup = $projectMSBuild.AddPropertyGroup()
$propertyGroup.Label = $scLabel

$relPathToToolsFolder = ComputeRelativePathToTargetsFile -startPath ($projItem = Get-Item $project.FullName) -targetPath (Get-Item ("{0}\tools\" -f $rootPath))
$propertyGroup.AddProperty('WebCompilerToolsPath', ('$([System.IO.Path]::GetFullPath( $(MSBuildProjectDirectory)\{0}\))') -f $relPathToToolsFolder);

$propEnableNuGetImport = $propertyGroup.AddProperty('WebCompiler_EnableImportFromNuGet', 'true');
$propEnableNuGetImport.Condition = ' ''$(WebCompiler_EnableImportFromNuGet)''=='''' ';

$importStmt = ('$([System.IO.Path]::GetFullPath( $(MSBuildProjectDirectory)\WebCompiler.targets ))')
$propNuGetImportPath = $propertyGroup.AddProperty('WebCompiler_NuGetImportPath', "$importStmt");
$propNuGetImportPath.Condition = ' ''$(WebCompiler_NuGetImportPath)''=='''' ';

$propImport = $propertyGroup.AddProperty('WebCompilerTargets', '$(WebCompiler_NuGetImportPath)');
$propImport.Condition = ' ''$(WebCompiler_EnableImportFromNuGet)''==''true'' and Exists(''$(WebCompiler_NuGetImportPath)'') ';

AddImportElementIfNotExists -projectRootElement $projectMSBuild

$projectMSBuild.Save()